/*************************************************************************
 *
*    Used with ICCARM and AARM.
 *
 *    (c) Copyright IAR Systems 2008
 *
 *    File name   : main.c
 *    Description : Main module
 *
 *    History :
 *    1. Date        : 4, August 2008
 *       Author      : Stanimir Bonev
 *       Description : Create
 *
 *    2. Date        : 9, September 2009
 *       Author      : Frederick Hjrner
 *       Description : Updated the example description
 *
 * This example project shows how to use the IAR Embedded Workbench for ARM
 * to develop code for IAR-LPC-2478 board. It shows basic use of I/O,
 * timer, interrupt and LCD controllers.
 *
 * The IAR, NXP and Olimex logos appear on the LCD and the cursor
 * moves as the board moves(the acceleration sensor is used).
 *
 * Jumpers:
 *  EXT/JLINK  - depending of power source
 *  ISP_E      - unfilled
 *  RST_E      - unfilled
 *  BDS_E      - unfilled
 *  C/SC       - SC
 *
 * Note:
 *  After power-up the controller get clock from internal RC oscillator that
 * is unstable and may fail with J-Link auto detect, therefore adaptive clocking
 * should always be used. The adaptive clock can be select from menu:
 *  Project->Options..., section Debugger->J-Link/J-Trace  JTAG Speed - Adaptive.
 *
 * The LCD shares pins with Trace port. If ETM is enabled the LCD will not work.
 *
 *    $Revision: 34711 $
 **************************************************************************/
#include "includes.h"
#include "exttest.h"

#define TIMER0_TICK_PER_SEC   10
#define DLY_100US             500

extern Int32U SDRAM_BASE_ADDR;
volatile Boolean TickFlag;

#define LCD_VRAM_BASE_ADDR ((Int32U)&SDRAM_BASE_ADDR)

#pragma location="USB_DMA_RAM"
#pragma data_alignment=4
__no_init Int8U Lun0Buffer[512];

/*************************************************************************
 * Function Name: Timer0IntrHandler
 * Parameters: none
 *
 * Return: none
 *
 * Description: Timer 0 interrupt handler
 *
 *************************************************************************/
void Timer0IntrHandler (void)
{
  TickFlag = TRUE;
  // clear interrupt
  T0IR_bit.MR0INT = 1;
  VICADDRESS = 0;
}

void InitRTC(void)
{
	PCONP_bit.PCRTC = 1;
	
	CCR_bit.CTCRST = 1;
	CCR_bit.CTCRST = 0;
	CCR_bit.CTTEST = 0;
	CCR_bit.CLKSRC = 1;
	CCR_bit.CLKEN = 1;
}

/*************************************************************************
 * Function Name: DelayResolution100us
 * Parameters: Int32U Dly
 *
 * Return: none
 *
 * Description: Delay ~ (arg * 100us)
 *
 *************************************************************************/
void DelayResolution100us(Int32U Dly)
{
  for(; Dly; Dly--)
  {
    for(volatile Int32U j = DLY_100US; j; j--)
    {
    }
  }
}

/*************************************************************************
 * Function Name: main
 * Parameters: none
 *
 * Return: none
 *
 * Description: main
 *
 *************************************************************************/
int main(void)
{
	typedef Int32U ram_unit;
	int cursor_x = (C_GLCD_H_SIZE - CURSOR_H_SIZE)/2, cursor_y = (C_GLCD_V_SIZE - CURSOR_V_SIZE)/2;
	ToushRes_t XY_Touch;
	int	prev_secs;

  GLCD_Ctrl (FALSE);

  // Init GPIO
  GpioInit();

  // initialize button
  BUT1_FDIR = ~BUT1_MASK;

#ifndef SDRAM_DEBUG
  // MAM init
  MAMCR_bit.MODECTRL = 0;
  MAMTIM_bit.CYCLES  = 3;   // FCLK > 40 MHz
  MAMCR_bit.MODECTRL = 2;   // MAM functions fully enabled
  // Init clock
  InitClock();
  // SDRAM Init
  SDRAM_Init();
#endif // SDRAM_DEBUG

  // Init VIC
  VIC_Init();

  /* only enter test if button is pressed at reset */
  if( !(BUT1_FIO & BUT1_MASK) ) {
	TestExt();
	if( SDRAM_Test() == true )
	  printf("SDRAM test OK.");

    // mark success
    USB_D_LINK_LED_SEL = 0; // GPIO
    USB_D_LINK_LED_FSET = USB_D_LINK_LED_MASK;
    USB_D_LINK_LED_FDIR |= USB_D_LINK_LED_MASK;
	while(1);
  }

  // GLCD init
//#ifdef SDRAM_DEBUG
//  GLCD_Init (LogoPic.pPicStream, NULL);
//#else
  GLCD_Init (NULL, NULL);
//#endif

  GLCD_Cursor_Dis(0);
  GLCD_Copy_Cursor ((Int32U *)Cursor, 0, sizeof(Cursor)/sizeof(Int32U));
  GLCD_Cursor_Cfg(CRSR_FRAME_SYNC | CRSR_PIX_32);
  GLCD_Move_Cursor(cursor_x, cursor_y);
  GLCD_Cursor_En(0);

  // Init USB Link  LED
  USB_D_LINK_LED_FDIR = USB_D_LINK_LED_MASK;
  USB_D_LINK_LED_FSET = USB_D_LINK_LED_MASK;

  // Enable TIM0 clocks
  PCONP_bit.PCTIM0 = 1; // enable clock

  // Init Time0
  T0TCR_bit.CE = 0;     // counting  disable
  T0TCR_bit.CR = 1;     // set reset
  T0TCR_bit.CR = 0;     // release reset
  T0CTCR_bit.CTM = 0;   // Timer Mode: every rising PCLK edge
  T0MCR_bit.MR0I = 1;   // Enable Interrupt on MR0
  T0MCR_bit.MR0R = 1;   // Enable reset on MR0
  T0MCR_bit.MR0S = 0;   // Disable stop on MR0
  // set timer 0 period
  T0PR = 0;
  T0MR0 = SYS_GetFpclk(TIMER0_PCLK_OFFSET)/(TIMER0_TICK_PER_SEC);
  // init timer 0 interrupt
  T0IR_bit.MR0INT = 1;  // clear pending interrupt
  VIC_SetVectoredIRQ(Timer0IntrHandler,0,VIC_TIMER0);
  VICINTENABLE |= 1UL << VIC_TIMER0;
  T0TCR_bit.CE = 1;     // counting Enable

  	// Init touch screen
  TouchScrInit();

  InitRTC();

    // Init SCSI module
  ScsiInit();

  // LUNs Init
  LunInit(SD_DISK_LUN,SdDiskInit,SdGetDiskCtrlBkl,SdDiskIO);

  __enable_interrupt();

  // Init SD card driver
  // SCSI init push init SD driver message for an each LUN
  for(Int32U i = 0; i < SCSI_LUN_NUMB; i++)
  {
    // Implement LUNs messages
    if(LunImp(i))
    {
      USB_D_LINK_LED_FCLR = USB_D_LINK_LED_MASK;
    }
    else
    {
      USB_D_LINK_LED_FSET = USB_D_LINK_LED_MASK;
    }
  }

  // Soft connection enable
  USB_ConnectRes(TRUE);

  // RTC indication
  USB_D_LINK_LED_SEL = 0; // GPIO
  USB_D_LINK_LED_FSET = USB_D_LINK_LED_MASK;
  USB_D_LINK_LED_FDIR |= USB_D_LINK_LED_MASK;

  GLCD_Ctrl (TRUE);

  while(1)
  {
	// Implement LUNs messages
	for(Int32U i = 0; i < SCSI_LUN_NUMB; i++)
    {
        LunImp(i);
    }
	
    if(TouchGet(&XY_Touch))
    {
		// the following lines introduce software filtering to increase touchscreen stability
		// without this filter the cursor vibrates +- 1 pixel
		// to disable set the treshold value to 0
#define TOUCH_DEVIATION_TRESHOLD 1		
		if( abs((int)XY_Touch.X - cursor_x) > TOUCH_DEVIATION_TRESHOLD ||
			abs((int)XY_Touch.Y - cursor_y) > TOUCH_DEVIATION_TRESHOLD ) {

			cursor_x = XY_Touch.X;
			cursor_y = XY_Touch.Y;

			GLCD_Move_Cursor(cursor_x, cursor_y);
		}
	}

	// toggle led every second
	if (prev_secs != SEC_bit.SEC)
	{
		prev_secs = SEC_bit.SEC;
		if (prev_secs % 2)
		{
	        USB_D_LINK_LED_FCLR = USB_D_LINK_LED_MASK;
		}
		else
		{
			USB_D_LINK_LED_FSET = USB_D_LINK_LED_MASK;
		}
	}
	
  }
}


